@@ -107,37 +107,58 @@ type Result<T, E = Error> = std::result::Result<T, E>;
 /// ```
 #[derive(Debug)]
 pub struct QueueFile {
-    file: File,
+    inner: QueueFileInner,
     /// True when using the versioned header format. Otherwise use the legacy format.
     versioned: bool,
     /// The header length in bytes: 16 or 32.
     header_len: u64,
-    /// Cached file length. Always a power of 2.
-    file_len: u64,
-    /// Last seek value
-    last_seek: u64,
     /// Number of elements.
     elem_cnt: usize,
     /// Pointer to first (or eldest) element.
     first: Element,
     /// Pointer to last (or newest) element.
     last: Element,
+    /// Minimum number of bytes the file shrinks to.
+    capacity: u64,
     /// When true, removing an element will also overwrite data with zero bytes.
     /// It's true by default.
     overwrite_on_remove: bool,
+    /// When true, skips header update upon adding.
+    /// It's false by default.
+    skip_write_header_on_add: bool,
+    /// Write buffering.
+    write_buf: Vec<u8>,
+}
+
+#[derive(Debug)]
+struct QueueFileInner {
+    file: File,
+    /// Cached file length. Always a power of 2.
+    file_len: u64,
+    /// Last seek value
+    last_seek: Option<u64>,
+    /// Buffer used by `transfer` function.
+    transfer_buf: Option<Box<[u8]>>,
     /// When true, every write to file will be followed by `sync_data()` call.
     /// It's true by default.
     sync_writes: bool,
-    /// Buffer used by `transfer` function.
-    transfer_buf: Option<Box<[u8]>>,
+}
+
+impl Drop for QueueFile {
+    fn drop(&mut self) {
+        if self.skip_write_header_on_add {
+            let _ = self.sync_header();
+        }
+    }
 }
 
 impl QueueFile {
+    const BLOCK_LENGTH: u64 = 4096;
     const INITIAL_LENGTH: u64 = 4096;
     const VERSIONED_HEADER: u32 = 0x8000_0001;
     const ZEROES: [u8; 4096] = [0; 4096];
 
-    fn init(path: &Path, force_legacy: bool) -> Result<()> {
+    fn init(path: &Path, force_legacy: bool, capacity: u64) -> Result<()> {
         let tmp_path = path.with_extension(".tmp");
 
         // Use a temp file so we don't leave a partially-initialized file.
@@ -145,15 +166,15 @@ impl QueueFile {
             let mut file =
                 OpenOptions::new().read(true).write(true).create(true).open(&tmp_path)?;
 
-            file.set_len(QueueFile::INITIAL_LENGTH)?;
+            file.set_len(capacity)?;
 
             let mut buf = BytesMut::with_capacity(16);
 
             if force_legacy {
-                buf.put_u32(QueueFile::INITIAL_LENGTH as u32);
+                buf.put_u32(capacity as u32);
             } else {
                 buf.put_u32(QueueFile::VERSIONED_HEADER);
-                buf.put_u64(QueueFile::INITIAL_LENGTH);
+                buf.put_u64(capacity);
             }
 
             file.write_all(buf.as_ref())?;
@@ -165,19 +186,23 @@ impl QueueFile {
         Ok(())
     }
 
+    pub fn with_capacity<P: AsRef<Path>>(path: P, capacity: u64) -> Result<QueueFile> {
+        Self::open_internal(path, true, false, capacity)
+    }
+
     pub fn open<P: AsRef<Path>>(path: P) -> Result<QueueFile> {
-        Self::open_internal(path, true, false)
+        Self::with_capacity(path, QueueFile::INITIAL_LENGTH)
     }
 
     pub fn open_legacy<P: AsRef<Path>>(path: P) -> Result<QueueFile> {
-        Self::open_internal(path, true, true)
+        Self::open_internal(path, true, true, QueueFile::INITIAL_LENGTH)
     }
 
     fn open_internal<P: AsRef<Path>>(
-        path: P, overwrite_on_remove: bool, force_legacy: bool,
+        path: P, overwrite_on_remove: bool, force_legacy: bool, capacity: u64,
     ) -> Result<QueueFile> {
         if !path.as_ref().exists() {
-            QueueFile::init(path.as_ref(), force_legacy)?;
+            QueueFile::init(path.as_ref(), force_legacy, capacity)?;
         }
 
         let mut file = OpenOptions::new().read(true).write(true).open(path)?;
@@ -249,19 +274,30 @@ impl QueueFile {
         file.rewind()?;
 
         let mut queue_file = QueueFile {
-            file,
+            inner: QueueFileInner {
+                file,
+                file_len,
+                last_seek: Some(0),
+                transfer_buf: Some(
+                    vec![0u8; QueueFileInner::TRANSFER_BUFFER_SIZE].into_boxed_slice(),
+                ),
+                sync_writes: cfg!(not(test)),
+            },
             versioned,
             header_len,
-            file_len,
-            last_seek: 0,
             elem_cnt,
             first: Element::EMPTY,
             last: Element::EMPTY,
+            capacity,
             overwrite_on_remove,
-            sync_writes: cfg!(not(test)),
-            transfer_buf: Some(vec![0u8; Self::TRANSFER_BUFFER_SIZE].into_boxed_slice()),
+            skip_write_header_on_add: false,
+            write_buf: Vec::new(),
         };
 
+        if file_len < capacity {
+            queue_file.inner.sync_set_len(queue_file.capacity)?;
+        }
+
         queue_file.first = queue_file.read_element(first_pos)?;
         queue_file.last = queue_file.read_element(last_pos)?;
 
@@ -280,12 +316,22 @@ impl QueueFile {
 
     /// Returns true if every write to file will be followed by `sync_data()` call.
     pub fn get_sync_writes(&self) -> bool {
-        self.sync_writes
+        self.inner.sync_writes
     }
 
     /// If set to true every write to file will be followed by `sync_data()` call.
     pub fn set_sync_writes(&mut self, value: bool) {
-        self.sync_writes = value
+        self.inner.sync_writes = value
+    }
+
+    /// Returns true if skips header update upon adding enabled.
+    pub fn get_skip_write_header_on_add(&self) -> bool {
+        self.skip_write_header_on_add
+    }
+
+    /// If set to true skips header update upon adding.
+    pub fn set_skip_write_header_on_add(&mut self, value: bool) {
+        self.skip_write_header_on_add = value
     }
 
     /// Returns true if this queue contains no entries.
@@ -300,7 +346,67 @@ impl QueueFile {
 
     /// Synchronizes the underlying file, look at [File::sync_all] doc for more info.
     pub fn sync_all(&mut self) -> Result<()> {
-        Ok(self.file.sync_all()?)
+        if self.skip_write_header_on_add {
+            self.sync_header()?;
+        }
+
+        Ok(self.inner.file.sync_all()?)
+    }
+
+    pub fn add_n(
+        &mut self, elems: impl IntoIterator<Item = impl AsRef<[u8]>> + Clone,
+    ) -> Result<()> {
+        let (count, total_len) = elems
+            .clone()
+            .into_iter()
+            .fold((0, 0), |(c, l), elem| (c + 1, l + Element::HEADER_LENGTH + elem.as_ref().len()));
+
+        if count == 0 {
+            return Ok(());
+        }
+
+        ensure!(self.elem_cnt + count < i32::max_value() as usize, TooManyElementsSnafu {});
+        self.expand_if_necessary(total_len as u64)?;
+
+        let was_empty = self.is_empty();
+        let mut pos = if was_empty {
+            self.header_len
+        } else {
+            self.wrap_pos(self.last.pos + Element::HEADER_LENGTH as u64 + self.last.len as u64)
+        };
+
+        let mut first_added = None;
+        let mut last_added = None;
+        self.write_buf.clear();
+
+        for elem in elems {
+            let elem = elem.as_ref();
+            let len = elem.len();
+            ensure!(len <= i32::max_value() as usize, ElementTooBigSnafu {});
+
+            if first_added.is_none() {
+                first_added = Some(Element::new(pos, len));
+            }
+            last_added = Some(Element::new(pos, len));
+
+            self.write_buf.extend(&(len as u32).to_be_bytes());
+            self.write_buf.extend(elem);
+
+            pos = self.wrap_pos(pos + Element::HEADER_LENGTH as u64 + len as u64);
+        }
+
+        let first_added = first_added.unwrap();
+        self.ring_write_buf(first_added.pos)?;
+
+        if was_empty {
+            self.first = first_added;
+        }
+        self.last = last_added.unwrap();
+
+        self.write_header(self.file_len(), self.elem_cnt + count, self.first.pos, self.last.pos)?;
+        self.elem_cnt += count;
+
+        Ok(())
     }
 
     /// Adds an element to the end of the queue.
@@ -311,7 +417,7 @@ impl QueueFile {
 
         ensure!(len <= i32::max_value() as usize, ElementTooBigSnafu {});
 
-        self.expand_if_necessary(len)?;
+        self.expand_if_necessary((Element::HEADER_LENGTH + len) as u64)?;
 
         // Insert a new element after the current last element.
         let was_empty = self.is_empty();
@@ -323,15 +429,20 @@ impl QueueFile {
 
         let new_last = Element::new(pos, len);
 
-        // Write length.
-        self.ring_write(new_last.pos, &(len as u32).to_be_bytes())?;
+        self.write_buf.clear();
+        // Add length.
+        self.write_buf.extend(&(len as u32).to_be_bytes());
+        // Add data.
+        self.write_buf.extend(buf);
 
-        // Write data.
-        self.ring_write(new_last.pos + Element::HEADER_LENGTH as u64, buf)?;
+        // Make actual write.
+        self.ring_write_buf(new_last.pos)?;
 
-        // Commit the addition. If was empty, first == last.
-        let first_pos = if was_empty { new_last.pos } else { self.first.pos };
-        self.write_header(self.file_len, self.elem_cnt + 1, first_pos, new_last.pos)?;
+        if !self.skip_write_header_on_add {
+            // Commit the addition. If was empty, first == last.
+            let first_pos = if was_empty { new_last.pos } else { self.first.pos };
+            self.write_header(self.file_len(), self.elem_cnt + 1, first_pos, new_last.pos)?;
+        }
         self.last = new_last;
         self.elem_cnt += 1;
 
@@ -391,7 +502,7 @@ impl QueueFile {
         }
 
         // Commit the header.
-        self.write_header(self.file_len, self.elem_cnt - n, new_first_pos, self.last.pos)?;
+        self.write_header(self.file_len(), self.elem_cnt - n, new_first_pos, self.last.pos)?;
         self.elem_cnt -= n;
         self.first = Element::new(new_first_pos, new_first_len);
 
@@ -405,22 +516,32 @@ impl QueueFile {
     /// Clears this queue. Truncates the file to the initial size.
     pub fn clear(&mut self) -> Result<()> {
         // Commit the header.
-        self.write_header(QueueFile::INITIAL_LENGTH, 0, 0, 0)?;
+        self.write_header(self.capacity, 0, 0, 0)?;
 
         if self.overwrite_on_remove {
-            self.seek(self.header_len)?;
-            let len = QueueFile::INITIAL_LENGTH - self.header_len;
-            self.write(&QueueFile::ZEROES[..len as usize])?;
+            self.inner.seek(self.header_len)?;
+            let first_block = self.capacity.min(Self::BLOCK_LENGTH) - self.header_len;
+            self.inner.write(&QueueFile::ZEROES[..first_block as usize])?;
+
+            if let Some(left) = self.capacity.checked_sub(Self::BLOCK_LENGTH) {
+                for _ in 0..left / Self::BLOCK_LENGTH {
+                    self.inner.write(&QueueFile::ZEROES)?;
+                }
+                let tail = left % Self::BLOCK_LENGTH;
+                if tail != 0 {
+                    self.inner.write(&QueueFile::ZEROES[..tail as usize])?;
+                }
+            }
         }
 
         self.elem_cnt = 0;
         self.first = Element::EMPTY;
         self.last = Element::EMPTY;
 
-        if self.file_len > QueueFile::INITIAL_LENGTH {
-            self.sync_set_len(QueueFile::INITIAL_LENGTH)?;
+        if self.file_len() > self.capacity {
+            self.inner.sync_set_len(self.capacity)?;
         }
-        self.file_len = QueueFile::INITIAL_LENGTH;
+        self.inner.file_len = self.capacity;
 
         Ok(())
     }
@@ -435,7 +556,7 @@ impl QueueFile {
     /// Returns the amount of bytes used by the backed file.
     /// Always >= [Self::used_bytes].
     pub fn file_len(&self) -> u64 {
-        self.file_len
+        self.inner.file_len
     }
 
     /// Returns the amount of bytes used by the queue.
@@ -450,13 +571,17 @@ impl QueueFile {
                 + self.header_len
         } else {
             // tail < head. The queue wraps.
-            self.last.pos + Element::HEADER_LENGTH as u64 + self.last.len as u64 + self.file_len
+            self.last.pos + Element::HEADER_LENGTH as u64 + self.last.len as u64 + self.file_len()
                 - self.first.pos
         }
     }
 
     fn remaining_bytes(&self) -> u64 {
-        self.file_len - self.used_bytes()
+        self.file_len() - self.used_bytes()
+    }
+
+    fn sync_header(&mut self) -> io::Result<()> {
+        self.write_header(self.file_len(), self.size(), self.first.pos, self.last.pos)
     }
 
     /// Writes header atomically. The arguments contain the updated values. The struct member fields
@@ -493,8 +618,8 @@ impl QueueFile {
             header_buf.put_i32(last_pos as i32);
         }
 
-        self.seek(0)?;
-        self.write(&header.as_ref()[..self.header_len as usize])
+        self.inner.seek(0)?;
+        self.inner.write(&header.as_ref()[..self.header_len as usize])
     }
 
     fn read_element(&mut self, pos: u64) -> io::Result<Element> {
@@ -510,39 +635,38 @@ impl QueueFile {
 
     /// Wraps the position if it exceeds the end of the file.
     fn wrap_pos(&self, pos: u64) -> u64 {
-        if pos < self.file_len {
-            pos
-        } else {
-            self.header_len + pos - self.file_len
-        }
+        if pos < self.file_len() { pos } else { self.header_len + pos - self.file_len() }
     }
 
     /// Writes `n` bytes from buffer to position in file. Automatically wraps write if position is
     /// past the end of the file or if buffer overlaps it.
-    fn ring_write(&mut self, pos: u64, buf: &[u8]) -> io::Result<()> {
+    fn ring_write_buf(&mut self, pos: u64) -> io::Result<()> {
         let pos = self.wrap_pos(pos);
 
-        if pos + buf.len() as u64 <= self.file_len {
-            self.seek(pos)?;
-            self.write(buf)
+        if pos + self.write_buf.len() as u64 <= self.file_len() {
+            self.inner.seek(pos)?;
+            self.inner.write(&self.write_buf)
         } else {
-            let before_eof = (self.file_len - pos) as usize;
+            let before_eof = (self.file_len() - pos) as usize;
 
-            self.seek(pos)?;
-            self.write(&buf[..before_eof])?;
-            self.seek(self.header_len)?;
-            self.write(&buf[before_eof..])
+            self.inner.seek(pos)?;
+            self.inner.write(&self.write_buf[..before_eof])?;
+            self.inner.seek(self.header_len)?;
+            self.inner.write(&self.write_buf[before_eof..])
         }
     }
 
     fn ring_erase(&mut self, pos: u64, n: usize) -> io::Result<()> {
         let mut pos = pos;
-        let mut len = n as i64;
+        let mut len = n;
 
+        self.write_buf.clear();
+        self.write_buf.extend(QueueFile::ZEROES);
         while len > 0 {
-            let chunk_len = min(len, QueueFile::ZEROES.len() as i64);
+            let chunk_len = min(len, QueueFile::ZEROES.len());
+            self.write_buf.truncate(chunk_len);
 
-            self.ring_write(pos, &QueueFile::ZEROES[..chunk_len as usize])?;
+            self.ring_write_buf(pos)?;
 
             len -= chunk_len;
             pos += chunk_len as u64;
@@ -555,38 +679,37 @@ impl QueueFile {
     fn ring_read(&mut self, pos: u64, buf: &mut [u8]) -> io::Result<()> {
         let pos = self.wrap_pos(pos);
 
-        if pos + buf.len() as u64 <= self.file_len {
-            self.seek(pos)?;
-            self.read(buf)
+        if pos + buf.len() as u64 <= self.file_len() {
+            self.inner.seek(pos)?;
+            self.inner.read(buf)
         } else {
-            let before_eof = (self.file_len - pos) as usize;
+            let before_eof = (self.file_len() - pos) as usize;
 
-            self.seek(pos)?;
-            self.read(&mut buf[..before_eof])?;
-            self.seek(self.header_len)?;
-            self.read(&mut buf[before_eof..])
+            self.inner.seek(pos)?;
+            self.inner.read(&mut buf[..before_eof])?;
+            self.inner.seek(self.header_len)?;
+            self.inner.read(&mut buf[before_eof..])
         }
     }
 
     /// If necessary, expands the file to accommodate an additional element of the given length.
-    fn expand_if_necessary(&mut self, data_len: usize) -> io::Result<()> {
-        let elem_len = Element::HEADER_LENGTH + data_len;
+    fn expand_if_necessary(&mut self, data_len: u64) -> io::Result<()> {
         let mut rem_bytes = self.remaining_bytes();
 
-        if rem_bytes >= elem_len as u64 {
+        if rem_bytes >= data_len {
             return Ok(());
         }
 
-        let mut prev_len = self.file_len;
+        let mut prev_len = self.file_len();
         let mut new_len = prev_len;
 
-        while rem_bytes < elem_len as u64 {
+        while rem_bytes < data_len {
             rem_bytes += prev_len;
             new_len = prev_len << 1;
             prev_len = new_len;
         }
 
-        self.sync_set_len(new_len)?;
+        self.inner.sync_set_len(new_len)?;
 
         // // Calculate the position of the tail end of the data in the ring buffer
         let end_of_last_elem =
@@ -597,17 +720,17 @@ impl QueueFile {
         if end_of_last_elem <= self.first.pos {
             count = end_of_last_elem - self.header_len;
 
-            let write_pos = self.seek(self.file_len)?;
-            self.transfer(self.header_len, write_pos, count)?;
+            let write_pos = self.inner.seek(self.file_len())?;
+            self.inner.transfer(self.header_len, write_pos, count)?;
         }
 
         // Commit the expansion.
         if self.last.pos < self.first.pos {
-            let new_last_pos = self.file_len + self.last.pos - self.header_len;
+            let new_last_pos = self.file_len() + self.last.pos - self.header_len;
             self.last = Element::new(new_last_pos, self.last.len);
         }
 
-        self.file_len = new_len;
+        self.inner.file_len = new_len;
 
         if self.overwrite_on_remove {
             self.ring_erase(self.header_len, count as usize)?;
@@ -618,32 +741,40 @@ impl QueueFile {
 }
 
 // I/O Helpers
-impl QueueFile {
+impl QueueFileInner {
     const TRANSFER_BUFFER_SIZE: usize = 128 * 1024;
 
     fn seek(&mut self, pos: u64) -> io::Result<u64> {
-        if pos == self.last_seek {
+        if Some(pos) == self.last_seek {
             Ok(pos)
         } else {
-            self.last_seek = self.file.seek(SeekFrom::Start(pos))?;
-            Ok(pos)
+            let res = self.file.seek(SeekFrom::Start(pos));
+            self.last_seek = res.as_ref().ok().copied();
+            res
         }
     }
 
     fn read(&mut self, buf: &mut [u8]) -> io::Result<()> {
-        self.last_seek += buf.len() as u64;
-        self.file.read_exact(buf)
+        if let Some(seek) = &mut self.last_seek {
+            *seek += buf.len() as u64;
+        }
+        let r = self.file.read_exact(buf);
+        if r.is_err() {
+            self.last_seek = None;
+        }
+        r
     }
 
     fn write(&mut self, buf: &[u8]) -> io::Result<()> {
-        self.last_seek += buf.len() as u64;
-        self.file.write_all(buf)?;
-
-        if self.sync_writes {
-            self.file.sync_data()
-        } else {
-            Ok(())
+        if let Some(seek) = &mut self.last_seek {
+            *seek += buf.len() as u64;
         }
+        if let Err(err) = self.file.write_all(buf) {
+            self.last_seek = None;
+            return Err(err);
+        }
+
+        if self.sync_writes { self.file.sync_data() } else { Ok(()) }
     }
 
     fn transfer_inner(
@@ -776,8 +907,11 @@ mod tests {
 
     fn gen_rand_file_name() -> String {
         let mut rng = thread_rng();
-        let mut file_name =
-            iter::repeat(()).map(|()| rng.sample(Alphanumeric)).map(char::from).take(16).collect::<String>();
+        let mut file_name = iter::repeat(())
+            .map(|()| rng.sample(Alphanumeric))
+            .map(char::from)
+            .take(16)
+            .collect::<String>();
 
         file_name.push_str(".qf");
 
@@ -785,20 +919,28 @@ mod tests {
     }
 
     fn new_queue_file(overwrite_on_remove: bool) -> (PathBuf, QueueFile) {
-        new_queue_file_ex(overwrite_on_remove, false)
+        new_queue_file_ex(overwrite_on_remove, false, None)
     }
 
     fn new_legacy_queue_file(overwrite_on_remove: bool) -> (PathBuf, QueueFile) {
-        new_queue_file_ex(overwrite_on_remove, true)
+        new_queue_file_ex(overwrite_on_remove, true, None)
     }
 
-    fn new_queue_file_ex(overwrite_on_remove: bool, force_legacy: bool) -> (PathBuf, QueueFile) {
+    fn new_queue_file_ex(
+        overwrite_on_remove: bool, force_legacy: bool, capacity: Option<u64>,
+    ) -> (PathBuf, QueueFile) {
         let file_name = gen_rand_file_name();
         let path = Path::new(file_name.as_str());
         let mut queue_file = if force_legacy {
+            assert!(capacity.is_none(), "capacity can be set only for non-legacy queue");
             QueueFile::open_legacy(path).unwrap()
         } else {
-            QueueFile::open(path).unwrap()
+            if let Some(capacity) = capacity {
+                QueueFile::with_capacity(path, capacity)
+            } else {
+                QueueFile::open(path)
+            }
+            .unwrap()
         };
 
         queue_file.set_overwrite_on_remove(overwrite_on_remove);
@@ -905,11 +1047,56 @@ mod tests {
         remove_file(&path).unwrap_or(());
     }
 
+    #[test]
+    fn queue_capacity_preserved() {
+        for is_overwrite in [false, true] {
+            eprintln!("is_overwrite = {is_overwrite}");
+
+            let initial_size = 517;
+            let (p, mut qf) = new_queue_file_ex(is_overwrite, false, Some(initial_size));
+
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size);
+
+            for i in 0u32..40 {
+                qf.add(&i.to_be_bytes()).unwrap();
+            }
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size);
+
+            for i in 0u32..40 {
+                qf.add(&i.to_be_bytes()).unwrap();
+            }
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size * 2);
+
+            qf.clear().unwrap();
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size);
+
+            remove_file(&p).unwrap_or(());
+        }
+    }
+
+    #[test]
+    fn existing_queue_extended_on_new_capacity() {
+        for is_overwrite in [false, true] {
+            eprintln!("is_overwrite = {is_overwrite}");
+            let initial_size = 200;
+            let (p, qf) = new_queue_file_ex(is_overwrite, false, Some(initial_size));
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size);
+
+            drop(qf);
+
+            let initial_size2 = 350;
+            let qf = QueueFile::with_capacity(&p, initial_size2).unwrap();
+            assert_eq!(std::fs::metadata(&p).unwrap().len(), initial_size2);
+            drop(qf);
+
+            remove_file(&p).unwrap_or(());
+        }
+    }
+
     fn add_rand_n_elems(
         q: &mut VecDeque<Box<[u8]>>, qf: &mut QueueFile, min_n: usize, max_n: usize,
         min_data_size: usize, max_data_size: usize,
-    ) -> usize
-    {
+    ) -> usize {
         let mut rng = thread_rng();
 
         let n = rng.gen_range(min_n..max_n);
@@ -948,8 +1135,7 @@ mod tests {
     fn simulate_use(
         path: &Path, mut qf: QueueFile, iters: usize, min_n: usize, max_n: usize,
         min_data_size: usize, max_data_size: usize, clear_prob: f64, reopen_prob: f64,
-    )
-    {
+    ) {
         let mut q: VecDeque<Box<[u8]>> = VecDeque::with_capacity(128);
 
         add_rand_n_elems(&mut q, &mut qf, min_n, max_n, min_data_size, max_data_size);
@@ -982,4 +1168,42 @@ mod tests {
         assert_eq!(q.is_empty(), true);
         assert_eq!(qf.is_empty(), true);
     }
+
+    #[test]
+    fn add_n_works() {
+        let (path, mut qf) = new_queue_file(true);
+
+        let data = vec![vec![1, 2, 3], vec![4, 5, 6]];
+
+        qf.add_n(data.iter()).unwrap();
+
+        let stored = qf.iter().map(Vec::from).collect::<Vec<_>>();
+        assert_eq!(data, stored);
+
+        qf.remove_n(1).unwrap();
+
+        let stored = qf.iter().map(Vec::from).collect::<Vec<_>>();
+        assert_eq!(&data[1..], stored);
+
+        qf.remove_n(1).unwrap();
+
+        let stored = qf.iter().map(Vec::from).collect::<Vec<_>>();
+        assert_eq!(Vec::<Vec<u8>>::new(), stored);
+
+        let data = vec![vec![1, 2, 3], vec![4, 5, 6], vec![7, 8]];
+
+        qf.add_n(data.iter()).unwrap();
+
+        let stored = qf.iter().map(Vec::from).collect::<Vec<_>>();
+        assert_eq!(data, stored);
+
+        let data: Vec<Vec<_>> = (0..=5).map(|i| vec![i, i, i, i, i, i, i, i, i, i]).collect();
+
+        qf.add_n(data.iter()).unwrap();
+
+        let stored = qf.iter().map(Vec::from).collect::<Vec<_>>();
+        assert_eq!(&data, &stored[3..]);
+
+        remove_file(&path).unwrap_or(());
+    }
 }
